%%%-----------------------------------------
%%%  
%%%  Curso EDO, 1er semestre 2016. U. Chile.
%%%  Profesor  :  Gino Montecinos
%%%  Auxiliares:  Danko Lobos y Felipe Matus
%%%  
%%%-----------------------------------------
%%%  Purpose: Phase portrait for general
%%%  2D ODE's.  The ODE is given by an
%%%  equation
%%%
%%%   X' = F(X),
%%%
%%%  where F(X) is a non-linear ecuation
%%%  in  2D.
%%%
%%% The user may modify:
%%%  - the function F(X);
%%%  - the interval in which initial conditions
%%%    are distribuited,  [a0,a1] x [b0,b1];
%%%  - the partition of [a0,a1], in terms of
%%%    NUM_PaternX;
%%%  - the partition of [b0,b1], in terms of
%%%    NUM_PaternY;
%%%
%%% Paremeters for the numerical method:
%%%  - the final time tEnd;
%%%  - Iterations of the numerical scheme;
%%%
%%%  The user also may set a particular
%%%  test by choosing; Test_type = 7.
%%%  The following  examples have been incorporated;
%%%  0 : Periodic behavior; circles and saddles   
%%%  1 : Nonlinear with circle and saddle points
%%%  2 : Saddle point
%%%  3 : Sink point
%%%  4 : Sink with repeated eigenvalues
%%%  5 : Clockwise circle
%%%  6 : Clockwise spiral Sink. 
%%%-------------------------------
clear all
close all

global Test_type




%%%----------------------------------
%%% To maintain point in a 2D plane.
hold on

%%%----------------------------
%%%  We set the maximum value of
%%%  coordinates to mantain the
%%%  graphic in a limited region in
%%%  the phase portrait.
boundL = 0 ;
boundR = 4.5;


%%%-----------------------------------------------
%%%  0 : Periodic behavior; circles and saddles   
%%%  1 : Nonlinear with circle and saddle points
%%%  2 : Saddle point
%%%  3 : Sink point
%%%  4 : Sink with repeated eigenvalues
%%%  5 : Clockwise circle
%%%  6 : Clockwise spiral Sink.
%%%  7 : User defined ODE.
%%% 
Test_type = 7 ;
%%%---------------------------------------

if(Test_type == 0)
  %% Periodic behavior; circles and saddles
  name ="Periodic behavior; circles and saddles."
  %%-----------------------------------
  %% Build a set of initial
  %% conditions distribuited
  %% uniformly.
  %%
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 20 ;
  NUM_PaternY = 20 ;

  %%---------------------------------------
  %% Definition of the rectangle in which
  %% initial conditions will be distribuited.
  %%
  %%  (x0,y0)\in [a0,a1] x [b0,b1]
  %%
  %% coordinate limits for x0
  a0 = -3/2 *pi;
  a1  = 3/2 *pi;
  %% coordinate limits for y0
  b0 = -3/2 *pi;
  b1 =  3/2 *pi;
end

if(Test_type == 1)
  %% Nonlinear ODE with circle and saddle points
  name ="Nonlinear with circle and saddle points.";
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 20 ;
  NUM_PaternY = 20 ;

  %% coordinate limits for x0
  a0 = -1;
  a1  = 3;
  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  2 ;

end

if( Test_type == 2 )
  %% Saddle point
  name = "Saddle point."; 
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 15 ;
  NUM_PaternY = 15 ;

  %% coordinate limits for x0
  a0 = -1;
  a1  = 1;
  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  1 ;


  boundL = 0 ;
  boundR = 1 ;
  
end

if( Test_type == 3)
  %%  Sink point
  name = "Sink point.";
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 5 ;
  NUM_PaternY = 5 ;

  %% coordinate limits for x0
  a0 = -1;
  a1  = 1;
  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  1 ;


  boundL = 0 ;
  boundR = 1 ;
  
end

if( Test_type == 4)
  %%  Sink with repeated eigenvalues
  name = "Sink with repeated eigenvalues.";
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 7 ;
  NUM_PaternY = 7 ;

  %% coordinate limits for x0
  a0 = -1;
  a1  = 1;
  %% coordinate limits for x0
  b0 = -1 ;
  b1 =  1 ;


  boundL = 0 ;
  boundR = 1 ;
  
end

if( Test_type == 5)
  %%  Clockwise circle
  name = "Clockwise circle.";
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 7 ;
  NUM_PaternY = 7 ;

  %% coordinate limits for x0
  a0 = -1;
  a1  = 1;

  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  1 ;


  boundL = 0 ;
  boundR = 1 ;
  
end

if( Test_type == 6)
  %%  Clsockwisw  spiral Sink. 
  name = "Clockwise spiral Sink."; 
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 7 ;
  NUM_PaternY = 7 ;
  
  %% coordinate limits for x0
  a0 = -1;
  a1  = 1;
  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  1 ;


  boundL = 0 ;
  boundR = 1 ;
  
end

if( Test_type == 7)
  %%  User defined ODE. 
  name = "User defined ODE."; 
  %%  NUM_PaternX : points along x direction
  %%  NUM_PaternY : points along y direction
  %%-----------------------------------
  NUM_PaternX = 9 ;
  NUM_PaternY = 9 ;

  %% coordinate limits for x0
  a0 = -3;
  a1  = 1;
  
  %% coordinate limits for y0
  b0 = -1 ;
  b1 =  2 ;


  boundL = 0 ;
  boundR = 4 ;
  
end
%%%---------------------------------------
%%%--------- End definition of parameter.
%%%---------------------------------------


%%%-------------------------------
%%%     computing the mesh size.
dx = (a1-a0)/NUM_PaternX;
dy = (b1-b0)/NUM_PaternY;

%%%------------------------------
%%% Parameters for the numerical
%%% scheme. Number of iterations
%%% and output time.
%%%------------------------------
%%% Final time of simulation.
tEnd = 10;  

%%%--------------------------------------
%%% Iterations for the numerical method.
iterations = 100;


%%%----------------------------
%%%  set the time step
dt = tEnd/(iterations);

%%%----------------------------------------
%%%  Display relevant iformation
%%%----------------------------------------
disp(sprintf("****************************************************"));
disp(sprintf(" Test:  %s ",name ));
disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Initial condition distribution in [a0,a1]x[b0,b1]:\n  %f << x0 << %f \n  %f << y0 << %f   ",a0,a1,b0,b1 ));
disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Partition of [a0,a1]= [%d,%d]:  %d ", a0,a1,NUM_PaternX));
disp(sprintf(" Partition of [b0,b1]= [%d,%d]:  %d ", b0,b1,NUM_PaternY));

disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Parameter boundR = %f, confines graphic to : \n [-%f , %f]x[-%f %f] ", ...
	     boundR,boundR,boundR,boundR,boundR));
disp(sprintf("----------------------------------------------------"));
disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Parameters for the numerical method of Heun"));
disp(sprintf("----------------------------------------------------"));
disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Simulation up to final time, tEnd:  %f ", tEnd));
disp(sprintf("----------------------------------------------------"));
disp(sprintf(" Simulation is carried out in:  %d iterations ", iterations));

disp(sprintf("****************************************************"));

%%%----------------------------------------
%%%  End display relevant iformation
%%%----------------------------------------

%%%------------------------------------
%%% time = 0;
%%% loop through every initial condition
%%% of initial conditions
for k  = 1 : NUM_PaternX
  for k2 = 1 : NUM_PaternY
    
    %%  Set the initial condition
    X0(1,1) = a0 + (k-1)* dx;
    X0(2,1) = b0 +(k2-1)* dy;
    
    %% Starting the ODE solver
    X = X0;
    %% At each time step we generate
    %% a new coordinate in the phase
    %% portrait.
    time = [];
    
    Xcoord  = [];
    Ycoord  = [];
    
    Xcoord=[ Xcoord X0(1,1)];    
    Ycoord=[ Ycoord X0(2,1)];
    
    time  =[time    dt*0];

    %% Plot only one point
    %% at each time step
    figure(1)
    plot(Xcoord,Ycoord)
    xlabel("x")
    ylabel("y")
    title(name)

    %%-----------------------------
    %% Start the numerical solution
    %% using Heun's method  
    for  i=1:iterations
      
      %% Predictor method for Heun 
      Xn = X +dt*F(X);

      %% Corrector method for Heun 
      for j=1:9
	Xn = X+ .5*dt*(F(Xn)+F(X) );
      end

      %% Each solution at a ginven time step
      %% is a new coordinate in the phase portrait
      X = Xn;

      
      %% Verify if the graphic is within the visulization
      %% range.
      if(    abs(X(1))< boundL  || abs(X(1)) > boundR)  	
	break;
      end
      
      if(   abs(X(2)) < boundL  || abs(X(2)) > boundR)  
	break;
      end            

      %% incorporate the new coordinate to the array of
      %% solutions.
      Xcoord=[ Xcoord X(1,1)];    
      Ycoord=[ Ycoord X(2,1)];   
      time  =[time    dt*i];
      
      %% plot the current history of solutions.
      plot(Xcoord,Ycoord)
      xlabel("x")
      ylabel("y")
      title(name)
      drawnow  
      
    end
    %%  End numerical solution
    %%-----------------------------   
  end
end
%%% End of initial condition  distribution 
disp(sprintf("Program ends normally."));      


